/***********************************************************************/
/*  This file is part of the uVision/ARM development tools             */
/*  Copyright KEIL ELEKTRONIK GmbH 2002-2005                           */
/***********************************************************************/
/*                                                                     */
/*  STARTUP.S:  Startup file for Philips LPC2000 device series         */
/*                                                                     */
/***********************************************************************/

/*
   This file has been heavily modified for the GNU-Toolchain by:
   Martin Thomas, Kaiserslautern, Germany
   <mthomas@rhrk.uni-kl.de>
   http://www.siwawi.arubi.uni-kl.de/avr_projects

   If it does not work for you: don't blame Keil or Philips.
*/

/*
 *  The STARTUP.S code is executed after CPU Reset. This file may be
 *  translated with the following SET symbols. In uVision these SET
 *  symbols are entered under Options - ASM - Set.
 */

/* Map Preprocessor definitions to assembler definitions/symbols */

// Standard definitions of Mode bits and Interrupt (I & F) flags in PSRs

	.set Mode_USR, 0x10
	.set Mode_FIQ, 0x11
	.set Mode_IRQ, 0x12
	.set Mode_SVC, 0x13
	.set Mode_ABT, 0x17
	.set Mode_UND, 0x1B
	.set Mode_SYS, 0x1F

	.set I_Bit, 0x80    /* when I bit is set, IRQ is disabled */
	.set F_Bit, 0x40    /* when F bit is set, FIQ is disabled */

/*
// <h> Stack Configuration (Stack Sizes in Bytes)
//   <o0> Undefined Mode      <0x0-0xFFFFFFFF:4>
//   <o1> Supervisor Mode     <0x0-0xFFFFFFFF:4>
//   <o2> Abort Mode          <0x0-0xFFFFFFFF:4>
//   <o3> Fast Interrupt Mode <0x0-0xFFFFFFFF:4>
//   <o4> Interrupt Mode      <0x0-0xFFFFFFFF:4>
//   <o5> User/System Mode    <0x0-0xFFFFFFFF:4>
// </h>
*/
	.set UND_Stack_Size, 0x00000080
	.set SVC_Stack_Size, 0x00000080
	.set ABT_Stack_Size, 0x00000080
	.set FIQ_Stack_Size, 0x00000080
	.set IRQ_Stack_Size, 0x00000200
	.set USR_Stack_Size, 0x00000800

.arm
.section .stack, "w"
.align 4
	.space (USR_Stack_Size+3)&~3  // Stack for User/System Mode
	.space (SVC_Stack_Size+3)&~3  // Stack for Supervisor Mode
	.space (IRQ_Stack_Size+3)&~3  // Stack for Interrupt Mode
	.space (FIQ_Stack_Size+3)&~3  // Stack for Fast Interrupt Mode
	.space (ABT_Stack_Size+3)&~3  // Stack for Abort Mode
	.space (UND_Stack_Size+3)&~3  // Stack for Undefined Mode
Top_Stack:


.text
.arm

.print "Vectors in section .vectorg -> .text"
.section .vectorg, "ax"

// Pre-defined interrupt handlers that may be directly
// overwritten by C interrupt functions
.extern IRQ_handler
.extern FIQ_handler
.extern SWI_handler
.extern UNDEF_handler
.extern PABT_handler
.extern DABT_handler


// Exception Vectors
// Mapped to Address 0.
// Absolute addressing mode must be used.

__Vectors:
	LDR		PC, Reset_Addr
	LDR		PC, Undef_Addr
	LDR		PC, SWI_Addr
	LDR		PC, PAbt_Addr
	LDR		PC, DAbt_Addr
	NOP				/* Reserved Vector (holds ISP checksum) */
	LDR		PC, [PC, #-0x0FF0]		/* Vector from VicVectAddr */
	LDR		PC, FIQ_Addr

Reset_Addr:	.word	Reset_Handler
Undef_Addr:	.word	UNDEF_handler		/* in main.c */
SWI_Addr:	.word	SoftwareInterrupt	/* in swi_handler.S */
PAbt_Addr:	.word	PABT_handler		/* in main.c */
DAbt_Addr:	.word	DABT_handler		/* in main.c */
IRQ_Addr:	.word	UNDEF_handler		/* (use VIC auto-vectoring) */
FIQ_Addr:	.word	FIQ_handler			/* in main.c */
			.word	0		/* rounds the vectors and ISR addresses to 64 bytes total */

.size   __Vectors, . - __Vectors


.arm
.section .init, "ax"
.global __startup
.func __startup
__startup:

Reset_Handler:
// Setup Stack for each mode
	LDR		R0, =Top_Stack

// Enter Undefined Instruction Mode and set its Stack Pointer
	MSR		CPSR_c, #Mode_UND|I_Bit|F_Bit
	MOV		SP, R0
	SUB		R0, R0, #UND_Stack_Size

// Enter Abort Mode and set its Stack Pointer
	MSR		CPSR_c, #Mode_ABT|I_Bit|F_Bit
	MOV		SP, R0
	SUB		R0, R0, #ABT_Stack_Size

// Enter FIQ Mode and set its Stack Pointer
	MSR		CPSR_c, #Mode_FIQ|I_Bit|F_Bit
	MOV		SP, R0
	SUB		R0, R0, #FIQ_Stack_Size

// Enter IRQ Mode and set its Stack Pointer
	MSR		CPSR_c, #Mode_IRQ|I_Bit|F_Bit
	MOV		SP, R0
	SUB		R0, R0, #IRQ_Stack_Size

// Enter Supervisor Mode and set its Stack Pointer
	MSR		CPSR_c, #Mode_SVC|I_Bit|F_Bit
	MOV		SP, R0
	SUB		R0, R0, #SVC_Stack_Size

// Enter User Mode and set its Stack Pointer
	MSR		CPSR_c, #Mode_USR /* Interrupts enabled */
//	MSR		CPSR_c, #Mode_USR|I_Bit|F_Bit /* Interrupts disabled */
	MOV		SP, R0

/* Relocate .data section (Copy from ROM to RAM) */
	LDR		R1, =_etext
	LDR		R2, =_data
	LDR		R3, =_edata
	CMP		R2, R3
	BEQ		DataIsEmpty
LoopRel:
	CMP		R2, R3
	LDRLO	R0, [R1], #4
	STRLO	R0, [R2], #4
	BLO		LoopRel
DataIsEmpty:

/* Clear .bss section (Zero init) */
	MOV		R0, #0
	LDR		R1, =__bss_start__
	LDR		R2, =__bss_end__
	CMP		R1,R2
	BEQ		BSSIsEmpty
LoopZI:
	CMP		R1, R2
	STRLO	R0, [R1], #4
	BLO		LoopZI
BSSIsEmpty:

// call C++ constructors of global objects
	LDR		r0, =__ctors_start__
	LDR		r1, =__ctors_end__
ctor_loop:
	CMP		r0, r1
	BEQ		ctor_end
	LDR		r2, [r0], #4
	STMFD	sp!, {r0-r1}
	MOV		lr, pc
	MOV		pc, r2
	LDMFD	sp!, {r0-r1}
	B		ctor_loop
ctor_end:

// Enter the C code
	LDR		R0,=main
	TST		R0,#1             // Bit-0 set: main is Thumb
	LDREQ	LR,=__exit_ARM    // ARM Mode
	LDRNE	LR,=__exit_THUMB  // Thumb Mode
	BX		R0

.size   __startup, . - __startup
.endfunc


.arm
.global __exit_ARM
.func __exit_ARM
__exit_ARM:
	B		__exit_ARM

.size   __exit_ARM, . - __exit_ARM
.endfunc


.thumb
.global __exit_THUMB
.func __exit_THUMB
__exit_THUMB:
	B		__exit_THUMB

.size   __exit_THUMB, . - __exit_THUMB
.endfunc


.end
