/******************************************************************************/
/*  This file is part of the uVision/ARM development tools                    */
/*  Copyright KEIL ELEKTRONIK GmbH 2002-2004                                  */
/******************************************************************************/
/*                                                                            */
/*  SERIAL.C:  Low Level Serial Routines                                      */
/*  modified and extended by Martin Thomas                                    */
/*                                                                            */
/******************************************************************************/

#include "LPC214x.h"
#include "common.h"

/*
 * Initialize Serial Interface UART0
 */
void cm_init_serial0(u_long baudrate)
{
	unsigned long Fdiv;

	PINSEL0 = 0x00000005;		/* Enable RxD0 and TxD0 */

	U0LCR = 0x83;				/* 8 bits, no Parity, 1 Stop bit */
	Fdiv = (Fpclk / 16) / baudrate;	/* calc division from baud rate */
	U0DLM = Fdiv / 256;
	U0DLL = Fdiv % 256;
	U0LCR = 0x03;				/* DLAB = 0 */
}

/*
 * Write character to Serial Port 0
 */
void cm_putchar(int ch)
{
	/*
	 * wait for the buffer empty
	 */
	while(!(U0LSR & 0x20))
		;

	U0THR = ch;
	return;
}

/*
 * Read character from Serial Port
 */
int cm_getchar(void)
{
	if(U0LSR & 0x01) {
		return(U0RBR);	/* something has come */
	}

	return -1;		/* nothing has come */
}
