/*****************************************************************************
 *   fiotest.c:  main C entry file for Philips LPC214x Family Microprocessors
 *
 *   Copyright(C) 2006, Philips Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2005.10.01  ver 1.00    Prelimnary version, first Release
 *
******************************************************************************/

#include "LPC214x.h"                        /* LPC21xx definitions */
#include "common.h"
#include "irq.h"
#include "timer.h"
#include "fio.h"
#include "uart.h"

#include <string.h>
static char Hello[]="Martin Thomas\r\n";

void TargetResetInit(void);

void IRQ_handler(void)   __attribute__ ((interrupt("IRQ")));
void FIQ_handler(void)   __attribute__ ((interrupt("FIQ")));
//void SWI_handler(void)   __attribute__ ((interrupt("SWI")));
void UNDEF_handler(void) __attribute__ ((interrupt("UNDEF")));
void PABT_handler(void)  __attribute__ ((interrupt("ABORT")));
void DABT_handler(void)  __attribute__ ((interrupt("ABORT")));

/*****************************************************************************
**   Main Function  main()
******************************************************************************/
int main (void)
{
	DWORD counter = 0;
	DWORD flip_port = REGULAR_PORT;

	TargetResetInit();

	cm_init_serial0(57600);			/* setup debug uart port */
	rprintf_devopen(cm_putchar);	/* init rprintf() */
	rprintf("rprintf: %s", "Hello from rprintf()\r\n");
while(1)
	;

	/********* The main Function is an endless loop ***********/
	init_VIC();
	init_timer();

	UARTInit(9600);		/* baud rate setting */
	U0IER = IER_THRE | IER_RLS;				/* Disable RBR */
	UARTSend( (BYTE*)Hello, strlen(Hello) );
	UART0Count = 0;
	U0IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */

	/* Initialize GPIO pins, default is REGULAR_PORT  */
	GPIOInit( REGULAR_PORT );

	/* Initialize timer for GPIO toggling timing */
	enable_timer( 0 );

	/* From the blink speed of the LED, it won't tell the difference between
	the regular port and fast port unless a scope is applied. This demo is
	show how the regular and fast port should be initialized and the level
	of the port can be changed. */
	while( 1 ) {
		if ( flip_port == REGULAR_PORT ) {
			if ( counter == 0 ) {
				GPIOInit( REGULAR_PORT ); /* only need to do once when port is
							switched. */
			}
			if ( timer_counter >= 0x20 * counter ) {
				IOSET1 = 1 << (16 + counter);
				counter++;
				if ( counter > 8 ) {
					counter = 0;
					timer_counter = 0;
					IOCLR1 = 0x00FF0000;
					flip_port = FAST_PORT;	/* switch port from regular to fast */
				} /* if counter > */
			} /* if timer_counter > */
		} /* regular_port */

		if ( flip_port == FAST_PORT )
		{
			if ( counter == 0 )
			{
			GPIOInit( FAST_PORT );		/* only need to do once when port is
							switched. */
			}
			if ( timer_counter >= 0x20 * counter )
			{
			FIO1SET = 1 << (16 + counter);
			counter++;
			if ( counter > 8 )
			{
				counter = 0;
				timer_counter = 0;
				FIO1CLR = 0x00FF0000;
				flip_port = REGULAR_PORT;	/* switch port from fast to regular */
			}
			}
		}
	}

	return 0;

}


/* ===============================================================
 * setup core hardware running conditions
 */
void TargetResetInit(void)
{
#ifdef __DEBUG_RAM
	MEMMAP = 0x2;                   /* set remap register */
#endif

#ifdef __DEBUG_FLASH
	MEMMAP = 0x1;                   /* set remap register */
#endif

#ifdef __IN_CHIP
	MEMMAP = 0x1;                   /* set remap register */
#endif

	/* Set system timers for each component */
	PLLCON = 1;
#if (Fpclk / (Fcclk / 4)) == 1
	VPBDIV = 0;
#endif
#if (Fpclk / (Fcclk / 4)) == 2
	VPBDIV = 2;
#endif
#if (Fpclk / (Fcclk / 4)) == 4
	VPBDIV = 1;
#endif

#if (Fcco / Fcclk) == 2
	PLLCFG = ((Fcclk / Fosc) - 1) | (0 << 5);
#endif
#if (Fcco / Fcclk) == 4
	PLLCFG = ((Fcclk / Fosc) - 1) | (1 << 5);
#endif
#if (Fcco / Fcclk) == 8
	PLLCFG = ((Fcclk / Fosc) - 1) | (2 << 5);
#endif
#if (Fcco / Fcclk) == 16
	PLLCFG = ((Fcclk / Fosc) - 1) | (3 << 5);
#endif
	PLLFEED = 0xaa;
	PLLFEED = 0x55;
	while((PLLSTAT & (1 << 10)) == 0);
	PLLCON = 3;
	PLLFEED = 0xaa;
	PLLFEED = 0x55;

	/* Set memory accelerater module*/
	MAMCR = 0;
#if Fcclk < 20000000
	MAMTIM = 1;
#else
#if Fcclk < 40000000
	MAMTIM = 2;
#else
	MAMTIM = 3;
#endif
#endif
	MAMCR = 2;

	/* Add your codes here */
	return;
}

/* ===============================================================
 * exception handlers
 */
void IRQ_handler(void)
{
	rprintf("!!IRQ\r\n");
	while(1)
		;		/* halt at here */
}

void FIQ_handler(void)
{
	rprintf("!!FIQ\r\n");
	while(1)
		;		/* halt at here */
}

void UNDEF_handler(void)
{
	rprintf("!!UNDEF\r\n");
	while(1)
		;		/* halt at here */
}

void PABT_handler(void)
{
	rprintf("!!PABT\r\n");
	while(1)
		;		/* halt at here */
}

void DABT_handler(void)
{
	rprintf("!!DABT\r\n");
	while(1)
		;		/* halt at here */
}
